<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco56200LancamentoDespesaFactory;

/**
 * Class Bco56200LancamentoDespesaService
 * @package Teknisa\FIN\Modules\MovimentacaoBancaria\Services
 */
class Bco56200LancamentoDespesaService {
    
    const INCLUSAO = 'N';
    const EXCLUSAO = 'S';
    const TIPO     = 'X';
    
    protected $entityManager;
    protected $environment;
    protected $newCode;
    protected $contabilizacaoService;
    protected $titulosAPagarFactory;

    /**
     * @param EntityManager                         $entityManager
     * @param Environment                           $environment
     * @param NewCode                               $newCode
     * @param FluxoDeCaixaService                   $fluxoDeCaixaService
     * @param ContabilizacaoService                 $contabilizacaoService
     * @param Bco56200LancamentoDespesaFactory      $bco56200LancamentoDespesaFactory
     */
    public function __construct(
        EntityManager $entityManager, Environment $environment, NewCode $newCode, FluxoDeCaixaService $fluxoDeCaixaService, ContabilizacaoService $contabilizacaoService, Bco56200LancamentoDespesaFactory $bco56200LancamentoDespesaFactory
    ){
        $this->entityManager                    = $entityManager;
        $this->environment                      = $environment;
        $this->newCode                          = $newCode;
        $this->fluxoDeCaixaService              = $fluxoDeCaixaService;
        $this->contabilizacaoService            = $contabilizacaoService;
        $this->bco56200LancamentoDespesaFactory = $bco56200LancamentoDespesaFactory;
    }

    /**
     * Salva Lançamento de Despesa Caixa
     * @param $row
     * @param $response
     * @throws \Exception
     */
    public function saveLancDespCaixa($row, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrOrgTrab();
            
            $saldocaixadisp = $this->bco56200LancamentoDespesaFactory->buscaSaldoCaixaDisp(
                $nrorg, $row['CDFILIAL'], $row['CDCENTCUST'], $row['DTLANCDESP']
            );
            
            if(floatval($saldocaixadisp['SALDODISP']) >= floatval($row['VRLANCDESP'])){
                $nrseqlancdesp = $this->newCode->getCodeByTableName('LANCDESPCAIXA', $nrorg);
                
                // Insere Lançamento de Despesa
                $lancdespcaixa = $this->bco56200LancamentoDespesaFactory->insereLancdespcaixa(
                    $nrorg, $nrseqlancdesp, $row['CDTIPODESP'], $row['DSLANCDESP'], $row['CDEMPRESA'],
                    $this->formataData($row['DTLANCDESP']), $row['CDFILIAL'], $row['CDCENTCUST'], $row['VRLANCDESP']
                );
                
                $this->entityManager->persist($lancdespcaixa);
                $this->entityManager->flush();
                
                // Atualiza Saldo Conta Corrente
                $this->bco56200LancamentoDespesaFactory->execProcAtualizaSaldoCaixa(
                    $row['CDFILIAL'], $row['CDCENTCUST'], $nrseqlancdesp, self::INCLUSAO
                );
                
                // Contabilização Conta Pagar
                $nrlanccont = $this->contabilizacaoService->contabilizaLancamentoControleBancario(
                    $row['CDEMPRESA'], null, null, self::TIPO, $row['IDCTBLDESP'], $row['CDCENTCUST'], $nrseqlancdesp
                );
            }else throw new \Exception("Operação bloqueada. Saldo indisponível para lançamento de despesa.<br/><br/>Saldo disponível: ".$saldocaixadisp['FORMATSALDODISP']);
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
    
    /**
     * Deleta Lançamento de Despesa Caixa
     * @param $row
     * @param $response
     * @throws \Exception
     */
    public function deleteLancDespCaixa($row, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            $nrorg = $this->environment->getNrOrgTrab();
            
            // Atualiza Saldo Conta Corrente
            $this->bco56200LancamentoDespesaFactory->execProcAtualizaSaldoCaixa(
                $row['CDFILIAL'], $row['CDCENTCUST'], $row['NRSEQLANCDESP'], self::EXCLUSAO
            );
            
            // Busca Lançamentos Contábil
            $lanccont = $this->bco56200LancamentoDespesaFactory->buscaLanccont(
                $nrorg, $row['CDEMPRESA'], $row['NRLANCCONT']
            );

            // Exclui Lançamentos Contábeis
            if ($lanccont)
                foreach ($lanccont as $lancctb)
                    $this->entityManager->remove($lancctb);
                    
            // Busca Lançamento de Despesa
            $lancdespcaixa = $this->bco56200LancamentoDespesaFactory->buscaLancdespcaixa(
                $nrorg, $row['NRSEQLANCDESP']
            );
            
            $this->entityManager->remove($lancdespcaixa);
            $this->entityManager->flush();
            
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
    
    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData(
        $data
    ){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);

        return $data;
    }

}